package org.jboss.security.ssl;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.Properties;
import javax.naming.InitialContext;
import javax.net.SocketFactory;
import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import org.jboss.logging.Logger;
import org.jboss.security.SecurityDomain;
import org.jboss.security.ssl.ClientSocketFactory;
import org.jboss.security.ssl.Context;

public class DomainSocketFactory extends SSLSocketFactory implements HandshakeCompletedListener {

   public static final String HANDSHAKE_COMPLETE_LISTENER = "org.jboss.security.ssl.HandshakeCompletedListener";
   private static Logger log = Logger.getLogger(DomainSocketFactory.class);
   private transient SecurityDomain securityDomain;
   private transient SSLContext sslCtx = null;
   private boolean wantsClientAuth = true;
   private boolean needsClientAuth = false;


   public DomainSocketFactory() {
   }

   public DomainSocketFactory(SecurityDomain securityDomain) throws IOException {
      if(securityDomain == null) {
         throw new IOException("The securityDomain may not be null");
      } else {
         this.securityDomain = securityDomain;
      }
   }

   public SecurityDomain getSecurityDomain() {
      return this.securityDomain;
   }

   public void setSecurityDomain(SecurityDomain securityDomain) {
      this.securityDomain = securityDomain;
   }

   public boolean isWantsClientAuth() {
      return this.wantsClientAuth;
   }

   public void setWantsClientAuth(boolean wantsClientAuth) {
      this.wantsClientAuth = wantsClientAuth;
   }

   public boolean isNeedsClientAuth() {
      return this.needsClientAuth;
   }

   public void setNeedsClientAuth(boolean needsClientAuth) {
      this.needsClientAuth = needsClientAuth;
   }

   public Socket createSocket(String serverHost, int serverPort) throws IOException, UnknownHostException {
      InetAddress serverAddr = InetAddress.getByName(serverHost);
      return this.createSocket((InetAddress)serverAddr, serverPort);
   }

   public Socket createSocket(String serverHost, int serverPort, InetAddress clientAddr, int clientPort) throws IOException, UnknownHostException {
      InetAddress serverAddr = InetAddress.getByName(serverHost);
      return this.createSocket((InetAddress)serverAddr, serverPort, clientAddr, clientPort);
   }

   public Socket createSocket(InetAddress serverAddr, int serverPort) throws IOException {
      return this.createSocket((InetAddress)serverAddr, serverPort, (InetAddress)null, 0);
   }

   public Socket createSocket(InetAddress serverAddr, int serverPort, InetAddress clientAddr, int clientPort) throws IOException {
      this.initSSLContext();
      SSLSocketFactory factory = this.sslCtx.getSocketFactory();
      SSLSocket socket = (SSLSocket)factory.createSocket(serverAddr, serverPort, clientAddr, clientPort);
      String[] supportedProtocols = socket.getSupportedProtocols();
      log.debug("Supported protocols: " + Arrays.asList(supportedProtocols));
      socket.setEnabledProtocols(supportedProtocols);
      socket.addHandshakeCompletedListener(this);
      socket.setNeedClientAuth(this.needsClientAuth);
      socket.setWantClientAuth(this.wantsClientAuth);
      return socket;
   }

   public Socket createSocket(Socket s, String host, int port, boolean autoClose) throws IOException {
      this.initSSLContext();
      SSLSocketFactory factory = this.sslCtx.getSocketFactory();
      SSLSocket socket = (SSLSocket)factory.createSocket(s, host, port, autoClose);
      socket.addHandshakeCompletedListener(this);
      return socket;
   }

   public String[] getDefaultCipherSuites() {
      String[] cipherSuites = new String[0];

      try {
         this.initSSLContext();
         SSLSocketFactory e = this.sslCtx.getSocketFactory();
         cipherSuites = e.getDefaultCipherSuites();
      } catch (IOException var3) {
         log.error("Failed to get default SSLSocketFactory", var3);
      }

      return cipherSuites;
   }

   public String[] getSupportedCipherSuites() {
      String[] cipherSuites = new String[0];

      try {
         this.initSSLContext();
         SSLSocketFactory e = this.sslCtx.getSocketFactory();
         cipherSuites = e.getSupportedCipherSuites();
      } catch (IOException var3) {
         log.error("Failed to get default SSLSocketFactory", var3);
      }

      return cipherSuites;
   }

   public static SocketFactory getDefault() {
      DomainSocketFactory ssf = null;

      try {
         InitialContext e = new InitialContext();
         SecurityDomain sd = (SecurityDomain)e.lookup("java:/jaas/other");
         ssf = new DomainSocketFactory(sd);
      } catch (Exception var3) {
         log.error("Failed to create default SocketFactory", var3);
      }

      return ssf;
   }

   public void handshakeCompleted(HandshakeCompletedEvent event) {
      Logger log = Logger.getLogger(ClientSocketFactory.class);
      if(log.isTraceEnabled()) {
         String e = event.getCipherSuite();
         SSLSession listener = event.getSession();
         String peerHost = listener.getPeerHost();
         log.debug("SSL handshakeCompleted, cipher=" + e + ", peerHost=" + peerHost);
      }

      try {
         Properties e1 = System.getProperties();
         HandshakeCompletedListener listener1 = (HandshakeCompletedListener)e1.get("org.jboss.security.ssl.HandshakeCompletedListener");
         if(listener1 != null) {
            listener1.handshakeCompleted(event);
         }
      } catch (Throwable var6) {
         log.debug("Failed to foward handshakeCompleted", var6);
      }

   }

   private void initSSLContext() throws IOException {
      if(this.sslCtx == null) {
         this.sslCtx = Context.forDomain(this.securityDomain);
      }
   }

}
